"Include this generated file after function definitions"


import BuiltIn
import os
import sys

BANDAID_LIST = ["rpc", "mcastRpc", "saveNvParam", "resetVm", "eraseImage",
    "writeChunk", "initVm", "rpcSourceAddr", "stdinMode", "chr", "cbusRd",
    "i2cRead", "spiRead", "spiXfer", "str", "vmStat", "updateDmxBuf"]

KNOBS_LIST = {"later": "WITH_CALL_LATER",
              "cbusRd": "WITH_CBUS",
              "cbusWr": "WITH_CBUS",
              "i2cRead": "WITH_I2C",
              "i2cWrite": "WITH_I2C",
              "i2cInit": "WITH_I2C",
              "getI2cResult": "WITH_I2C",
              "spiRead": "WITH_SPI",
              "spiXfer": "WITH_SPI",
              "spiWrite": "WITH_SPI",
              "spiInit": "WITH_SPI",}

def decorate(name):
    "Apply C-code 'pyWrapper' naming convention"
    return 'py' + name[0].upper() + name[1:]


if __name__ == "__main__":
    varname = 'const pyCFuncTbl'
    if len(sys.argv) > 1:
      os.chdir(os.path.realpath(sys.argv[1]))

    cf = BuiltIn.__cfuncs__.items()
    cf.sort(key=lambda x: x[1])

    buf = '// Generated by genCFuncWrapTbl.py\r\n#include "Knobs.h"\r\n\r\n'
    for nm,info in cf:
        bandaid = " BANDAID" if nm in BANDAID_LIST else ""
        if nm in KNOBS_LIST.iterkeys():
            buf += "#if %s == 0\r\n" % (KNOBS_LIST[nm])
            buf += "#define %s   pyBist\r\n" % (decorate(nm))
            buf += "#else\r\n"
        buf += "extern void %s(unsigned char nArgs)%s;\r\n" % (decorate(nm), bandaid)
        if nm in KNOBS_LIST.iterkeys():
            buf += "#endif\r\n"

    buf += "\r\ntypedef void (*PyFuncWrap)(unsigned char nArgs);\r\n\r\n"

    buf += "// If not a DMX build, make the pyBist() stub fill in as DMX stubs too\r\n"
    buf += "#ifndef WITH_DMX\r\n"
    buf += "#define pyUpdateDmxBuf   pyBist\r\n"
    buf += "#define pyGetDmxBuf      pyBist\r\n"
    buf += "#endif\r\n\r\n"

    buf += "DECL_ROM PyFuncWrap %s[] = {\r\n" % varname
    for nm,info in cf:
        buf += ''.join(["  %-15s,  // %i\r\n" % (decorate(nm),info)])
        if nm == "vmStat":
            vmStatId = info
    buf += "};\r\n"

    buf += "\r\n#define VM_STAT_ID   %d\r\n" % vmStatId

    buf += "\r\n#define NUM_BUILTINS %d\r\n" % len(cf)

    orig_contents = ''
    if os.path.isfile('pyCFuncTbl.c'):
        with open('pyCFuncTbl.c', 'rb') as fp:
            orig_contents = fp.read()

    if buf != orig_contents:
        with open('pyCFuncTbl.c', 'wb') as fp:
            fp.write(buf)
